<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/trunk/class/dao/extended/mx.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |       
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: mx.php 4535 2006-06-27 18:15:56Z claym $

/**
  * mx.php - contains the {@link Mx} class
  *
  * @author $Author: claym $
  * @filesource
  * @package    classes
  * @subpackage dao
  */

include_once DAODIR.'/Mx.php';

/**
  * An MX Record
  *
  * @package    classes
  * @subpackage dao
  */
class Mx extends DAO_Mx {

    // non-db variables
	var $hostname;

    // {{{ Mx (constructor)

    /**
      * Constructor
      *
      * @param  int MX ID (primary key)
      */
	function Mx ( $mx_id = 0 ) 
    {
        if ($mx_id) {
            $this->id = $mx_id;
            $this->get($this->id);
        }

    }
    // }}}

// Setters
    // {{{ setAll()
    function setAll() {
        $this->setName($this->name);
        $this->setEnabled($this->enabled);
    }
    // }}}
    // {{{ setName()
    function setName($nm = '')
    {
        if ($nm) {
            $domain = new Domain($this->domain);
            $this->name = $nm;
            $this->hostname = "$this->name.$domain->name";
        }
        return true;
    }
    // }}}
    // {{{ setEnabled()

    /**
      * @param  mixed   true/false, 'true'/'false', 1/0
      */
    function setEnabled($en = '')
    {
        if ($en === false || $en === 'false' || $en === 0) {
            $this->enabled = 0;
        } else {
            $this->enabled = 1;
        }

        return true;
    }

    // }}}

// Getters
    // {{{ _toArray()
    function _toArray()
    {
        $arr = array();

        $arr['hostname'] = $this->getHostname();

        return $arr;
    }
    // }}}
    // {{{ getHostname()
    function getHostname()
    {
        if (!$this->hostname) {
            $this->setName($this->name);
        }

        return $this->hostname;
    }
    // }}}

// Validators
    // {{{ validateName()
	//check for duplicate and valid names
	function validateName() {
		global $user, $conf;

        if (!isset($this->domain))
            return _("Domain value must be set");

        if (!isset($this->name))
            return _("Name must be entered");

		//first let's make sure there isn't already a mx record with the same name in the db already
		//  but it can have a different priority

        /* NOTE:  This seems worthless. . .why check once with "different priorities being ok",
         * then immediately checking again, ignoring IDs?  I'm skipping this 'different priorities'
         * one until I see reason to do otherwise
         */

        /*

        // "SELECT * FROM mx WHERE name LIKE '$name' AND domain='$domain->id' AND priority=$priority AND server LIKE '$server'"

        // create a domain object
        $this->domain_obj = new Domain($this->domain);

        $dbdo = DB_DataObject::Factory('mx');
        $dbdo->domain = $this->domain;
        $dbdo->priority = $this->priority;
        $dbdo->whereAdd("name LIKE '{$this->$name}'");
        $dbdo->whereAdd("server LIKE '{$this->$server}'");

        // check if the IDs match. . .if we're doing an update,
        // having the same name is ok
        $result = $dbdo->find(true);
        if ($result > 0 && $dbdo->id != $this->id) {
            $conf['error']->add_error('name', "{$this->name}.{$this->domain_obj->name} is already in use by an MX Record.");
            $isValid = false;
		}
        */

        // "SELECT * FROM mx WHERE name LIKE '$name' AND domain='$domain->id' AND server LIKE '$server'"
        $dbdo = DB_DataObject::Factory('mx');
        $dbdo->domain = $this->domain;
        $dbdo->whereAdd("name LIKE '$this->name'");
        $dbdo->whereAdd("server LIKE '$this->server'");
        $dbdo->whereAdd("id != '$this->id'");

		if ($dbdo->find() && !isset($this->id))
            return "$this->hostname --> $this->server "._("already exists");
	
		// original: if (!preg_match("/^[\w-]+$/", $this->name)) {
        if (!preg_match("/^\w+(\-\w+)*$/", $this->name))
            return "$this->hostname "._("is an invalid MX record");

        // "SELECT * FROM cname WHERE name LIKE '$name' AND domain='$domain->id'"
        $dbdo = DB_DataObject::Factory('cname');
        $dbdo->domain = $this->domain;
        $dbdo->whereAdd("name LIKE '$this->name'");

        if ($dbdo->find())
            return "$this->hostname "._("is already in use by a CNAME");

        return true;
	}
    // }}}
    // {{{ validateServer()
	//check for valid servername
	function validateServer() {
		global $user, $conf;	

		/* first lets make sure they aren't pointing a MX at a CNAME */
		$server_obj = split_hostname($this->server);

        // "SELECT * FROM cname,domain WHERE cname.domain=domain.id AND cname.name='$data->name' AND domain.name='$data->domain'"
        
        // set up the cname dbdo
        $dbdo_cname = DB_DataObject::Factory('cname');
        $dbdo_cname->name = $server_obj->name;

        // set up the domain dbdo
        $dbdo_domain = DB_DataObject::Factory('domain');
        $dbdo_domain->name = $server_obj->domain;

        // join them
        $dbdo_cname->joinAdd($dbdo_domain);

        if($dbdo_cname->find()) {
            $msg    = "$this->server "._("is already a CNAME.").' ';
            $msg    .= _("You can not have an MX record point to a CNAME");

            return $msg;
        }

		//now let's check and make sure that there are valid chr's in the server
		if (!Domain::is_valid_name($this->server))
            return "$this->server "._("is an invalid server name");

        return true;

	}
    // }}}
    // {{{ validatePriority()
	//check that we aren't setting this host to the same priority
	function validatePriority() {
		global $user, $conf;

        // "SELECT * FROM mx WHERE name='$name' AND domain='$domain->id' AND priority='$priority' AND id!='$this->id'"
        $dbdo = DB_DataObject::Factory('mx');
        $dbdo->name = $this->name;
        $dbdo->domain = $this->domain;
        $dbdo->priority = $this->priority;
        $dbdo->whereAdd("id!='$this->id'");

		if ($dbdo->find())
            return _("An MX record already exists with the same priority");

        return true;

	}
    // }}}
    // {{{ validateTtl()
    function validateTtl() {
        global $conf;

        if (!is_numeric($this->ttl))
            return _("TTL must be numeric");

        return true;
    }
    // }}}

// Built-in Methods
    // {{{ move_mx() 
	//move the mx record to a different zone
	function move_mx($mx_id,$zone_id) {
		global $conf;

        /* original task:
         * "UPDATE mx SET zone='$zone_id' WHERE id='$mx_id'"
         */

        // get the mx record to be updated
        $this->get('id', $mx_id);

        // set the new zone
        $this->zone = $zone_id;

        // update the record
        $this->update();

	} //move_mx
    // }}}
    // {{{ new_item() (deprecated)
	function new_item($name,$server,$domain,$priority,$zone,$ttl) {
		global $user, $conf;

        // "INSERT INTO mx SET id='', server='$server', name='$name',domain='$domain',priority='$priority',zone='$zone',ttl='$ttl'"
        $this->name = $name;
        $this->server = $server;
        $this->domain = $domain;
        $this->priority = $priority;
        $this->zone = $zone;
        $this->ttl = $ttl;
        
		return $this->insert();

	} // new_item
    // }}}

// Internal Methods
    // {{{ _history_text()

    function _history_text()
    {
        // : HOSTNAME --> SERVER
        return $this->getHostname() . ' --> ' . $this->getServer();
    }

    // }}}

}

?>
