<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/trunk/class/htdocs/Skin.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: Skin.php 4687 2006-08-01 22:32:37Z wenzel $

/**
  * Skin.php - contains {@link Skin} and the default maintain skin information
  *
  * @author $Author: wenzel $
  * @filesource
  * @package    skins
  */

require_once(MAINTAIN_ROOT."/includes.php");
require_once(HTDOCS."/includes.php");

/**
  * The base class for skins in Maintain
  *
  * @package    skins
  */
class Skin
{

//{{{   vars

    var $name; //name of the skin

//}}}
//{{{   Factory()

    /**
     *  This is the factory method that returns the appropriate skin for the user
     *  if it's not a real skin it defaults back to Maintain.  If the skin is just
     *  a collection of images or a stylesheet then they are used on the default
     *  skin which is specified in the config.
     *
     *  @param  String  $name   The name of the skin
     *
     *  @return Skin            Instance of the user's skin
     */
    function Factory($name)
    {
        global $conf, $images;

        // reference the path to the named skin
        $classpath = SKINDIR."/$name";

        // snag their includes
        $includes = $classpath.'/includes.php';

        //get the variable into scope.
        $skin = null; 

        //this is a skin directory
        if (is_dir($classpath) && file_exists($includes)) { 
            
            include_once($includes);
            
            //make instance of their layout class
            $skin = new $name(); 

            //set the name accordingly
            $skin->setName($name); 

        } elseif (!strcasecmp($name, 'maintain')) {
 
            $classpath = SKINDIR . "/maintain";
            $skin = new Maintain();
            $skin->setName('maintain');
        }
           
        //if that is a totally bogus skin name (directory doesn't exist)
        if(!$skin) {
             //make the default case
            $classpath = SKINDIR . "/{$conf['general']['default_skin']}";
            $skin = new $conf['general']['default_skin'];
            //If somehow the default skin is gone go to failsafe.
            if(!$skin) {
                $classpath = SKINDIR . "/maintain";
                $skin = new Maintain();
                $skin->setName('maintain');
            }
        }

        // grab the image set for this skin 
        //if (file_exists($classpath."/images.php"))
            include "$classpath/images.php";
        //else
          //  include SKINDIR."/maintain/images.php";
        foreach ($images as $key=>$image) {
            $images[$key] = "skins/$skin->name/$image";
        }
        $temp = $images;

        $default = $conf['general']['default_skin'];
        include SKINDIR . "/$default/images.php";
        foreach ($images as $key=>$image) {
            $images[$key] = "skins/$default/$image";
        }

        // merge the images arrays together
        $images = array_merge($images, $temp);

        return $skin;
    }
//}}}
//{{{   getLayout()

    /**
     *  This method gets the new layout.  Override this so it will load yours
     *  instead of the default.
     *
     *  @return PageWidget          A page widget for the specified state  
     */
    function getLayout()
    {
        global $conf;
        
        return new MaintainLayout($conf['info']['site_title']);
    }

//}}}
//{{{   setName()

    /**
     *  This method sets the skins name.  The name of a skin is used to determine
     *  image and CSS paths.  This is to allow someone to override just the images
     *  or just a CSS file.
     *  
     *  @param  string  $name   name of the skin
     */
    function setName($name)
    {
        /*Set the name of the skin for future notice*/
        $this->name = $name;
    }

//}}}
//{{{   display()

    /**
     *  The generate method actually generates the page and returns what needs
     *  to be printed.  First it sets the layout of the page, then it adds the
     *  content.  It then adds the CSS information from the addCSS method.  Finally
     *  it renders the page and returns the rendering for printing.
     *
     *  @final
     *
     *  @param  Container   $content    A phphtmllib Container object full of content
     *
     *  @return string                  The formatted HTML code
     */
    function display($content = '')
    {
        $lay = $this->getLayout();
        $lay->addContent($content);
        return $lay->render();
    }

//}}}
//{{{   getWidget()

    function getWidget()
    {
        return new MaintainWidget();
    }

//}}}
//{{{   amFactory()

    /**
     *  This is a factory method for action menus.  It may be called statically.
     *
     *  @static
     *  @param  int         $am     The type of Action Menu we're generating, 
     *                              there are constants defined for this param.
     *  @param  string      $object Then name of the type of object we're working 
     *                              with.
     *  @return Container   $amenu  A container that is going to be used as an action menu
     */
    function amFactory($am, $object, $id = "id")
    {
        global $skin;
        
        $amenu = $skin->getActionMenu();

        switch($am) {

        case AM_SHOW:
            $edit_link = html_a($_SERVER["SCRIPT_NAME"]."?action=edit&amp;$id=".$_REQUEST["$id"],
                                _("Edit"));
            $amenu->add($edit_link);
        case AM_EDIT:
            $del_link = html_a($_SERVER["SCRIPT_NAME"]."?action=delconfirm&amp;$id=".$_REQUEST["$id"],
                                _("Delete"));
            $show_link = html_a($_SERVER["SCRIPT_NAME"],_("Show").' '.$object.' '._(" List"));
            $amenu->add($del_link, $create_link, $show_link);

        case AM_LIST:
            $create_link = html_a($_SERVER["SCRIPT_NAME"]."?action=create",_("Create New"). ' '.$object);
            $amenu->add($create_link);
        default:
            break;
 
        }

        return $amenu;
    }

//}}}
//{{{   getActionMenu()

    /**
     *  This method gets an action menu, which is the bar that appears over data
     *  lists and over display objects.  It offers a quick way to change actions 
     *  and shortcuts to common tasks.
     *
     *  @return ULTag   $ul Unordered List that is our bar
     */
    function getActionMenu() {

        $ul = html_ul();
        $ul->set_id("actionmenu");

        return $ul;

    }

//}}}

}

/**
  * This class is a factory for various Maintain components
  *
  * @package    skins
  */
class Layout extends PageWidget
{

//{{{   Vars

    var $content;

//}}}
//{{{   Layout()

    /**
     *  Constructor
     *
     *  @param  String  $title  The title for the page
     *  @param  Const   $render The rendering method (defaults to HTML)
     *  @global $skin Skin object
     *  @global $images array of Maintain logos etc
     */
    function Layout($title, $render = HTML)
    {
        global $skin, $images;

        $this->PageWidget($title, $render);
        $this->content = container();

        //first we define the default CSS, incase they don't have a skin,
        //skins are disabled, or they simply didn't define something    
        $this->add_css_link(web_root()."skins/maintain/style.css");

        //Now lets load their CSS file, this will overwrite settings
        $this->add_css_link(web_root()."skins/".$skin->name."/style.css");

        // add Favicon if available
        if ($images[IMG_FAVICON]) {
            // both of these are needed due to standardization issues with MSIE
            $this->add_head_content(html_link($images[IMG_FAVICON], "shortcut icon", ""));
            $this->add_head_content(html_link($images[IMG_FAVICON], "icon", ""));
        }
    }

//}}}
//{{{   addContent()

    /**
     *  This method adds content to the layout.  Maintain expects this method 
     *  to exist.
     */
    function addContent($cont)
    {
        $this->content->add($cont);
    }

//}}}
//{{{   getMenu()

    /**
     *  This method is a getter that retrieves a menu object for this skin.
     *
     *  @return Mixed   An object to be used as the menu for Maintain.
     */
    function getMenu()
    {
        $ul = html_ul();
        $ul->set_id("menu");
        return $ul;
    }

//}}}

}

/**
  * The default Maintain Skin
  *
  * @package    skins
  * @subpackage maintain
  */
class Maintain extends Skin
{

//{{{   vars

    var $name = "maintain";

//}}}
//{{{   displayLogin()

    /**
     *  This function displays the login layout.  This is special to this skin.
     *
     *  @param  Mixed   $content    The content to be displayed in the login layout
     *
     *  @return String              The rendered page
     */
    function displayLogin($content)
    {
        $lay = new LoginLayout("Maintain Login");
        $lay->addContent($content);
        return $lay->render();
    }

// }}}
// {{{ displayInstall()

    /** This function displays the installation layout.  This is special to this skin.
     *
     *  @param  Mixed   $content    The content to be displayed in the install layout
     *
     * @return  String              The rendered page
     */
    function displayInstall($content)
    {
        $lay = new InstallLayout("Maintain Installation");
        $lay->addContent($content);
        return $lay->render();
    }

//}}}

}

/**
  * The bulk of the Maintain default Skin
  *
  * @package    skins
  * @subpackage maintain
  */
class MaintainLayout extends Layout 
{

//{{{   body_content()

    /**
     *  This method puts the page components together to form the actual page.
     */
    function body_content()
    {
        $this->add(html_comment("HEADER BLOCK BEGIN"));
        $this->add($this->header_block());
        $this->add(html_comment("HEADER BLOCK END"));

        $this->add(html_comment("MAIN BLOCK BEGIN"));
        $this->add($this->main_block());
        $this->add(html_comment("MAIN BLOCK END"));
          
        $this->add(html_comment("FOOTER BLOCK BEGIN"));
        $this->add($this->footer_block());
        $this->add(html_comment("FOOTER BLOCK END"));
    }

//}}}
//{{{   header_block()

    /**
     *  This method generates the head across the top of the HTDocs
     *
     *  @return HTMLTag     $header     The header HTML object  
     */
    function header_block()
    {
        global $conf, $user, $images;

        // create the link for non-CSS browsers to skip to content
        $hidden = html_a('#body-content', _("Skip to Content"), 'hidden');

        $zone_obj = new Zone($_SESSION['zone']);

        $header = html_div("pageheader");
        $zone_displayname = Module::apply_filter('zone_display',$zone_obj->name);
        
        $zone_message = ($_SESSION['zone']?container("You are currently working in zone: ",
                        html_b($zone_displayname)): "You are not currently working on a zone."); 

        $debug_msg = html_b(_("Debug Mode is Currently Enabled"));
        
        $user_displayname = Module::apply_filter('username_display', $user->username);
        $user_info = container(_("You are currently logged in as: "),
                                html_b($user_displayname), html_br(), $zone_message);
        
        if($conf['auth']['debug'])
            $user_info->add(html_br(),$debug_msg);

        $head_image_content = html_img($images[IMG_LOGO]);
        $head_image_content->set_id("logo");
        $head_image = html_a($conf['paths']['web_path'], $head_image_content);
        $head_info = html_div("headinfo");
        $head_info->add($user_info);
        
        $head_form_div = html_div("headform");
        $head_form = new FormProcessor(new HeaderForm());
        $head_form_div->add($head_form);
        
        $header->add($hidden, $head_image, $head_info, $head_form_div);
        
        return $header;
    }

//}}}
//{{{   main_block()

    /**
     *  This method breaks the main section of the page into its Nav and content
     *  components.
     *
     *  @return TABLEtag    $main   The table that is our main section.
     */
    function main_block()
    {
        // set up the table for the left/content columns
        $main = html_table();
        $main->set_class("content");

        // build the left column
        $left = html_td("left-block");
        $left->add($this->left_block());
      
        // build the content column
        $content = html_td("content-block");
        $content->add($this->content_block());

        // add them to the content
        $main->add_row($left, $content);
       
        return $main;
    }

//}}}
//{{{   left_block()
    
    /**
     *  This method builds the left block aka, nav bar.
     *
     *  @return HTMLTag     $div    The HTMLTag object that is the nav bar.    
     */
    function left_block()
    {
        global $skin, $user, $images;

        $menu =& $this->getMenu();
        
        $menu = Module::apply_filter('navlinks', $menu);
        
        return $menu;
    }

//}}}
//{{{   footer_block()

    /**
     *  This method constructs the footer for us.
     *
     *  @return HTMLTag $footer_div the constructed footer HTMLTag
     */
    function footer_block()
    {
        $table =  html_table();
        $table->set_class('footer');
        $table->add_row(new FooterWidget('footerlinks'));
        $table->add_row("&copy; 2005 OSUOSL");
        $table->add_row("Open Minds, Open Doors, Open Source");
        return $table;
    }

//}}}
//{{{   content_block()

    /**
     *  This function is what builds the content block.  This is a helper method.
     *
     *  @return the content for the object
     */
    function content_block()
    {
        return $this->content;
    }

//}}}

}

/**
  * The Default Menu for Maintain
  *
  * @package    skins
  * @subpackage maintain
  */
class MaintainWidget extends BaseWidget
{

//{{{   render()

    /**
     *  This method is responsible for rendering the HTML for the menu.
     *
     *  @param  int     $indent_level   This tells the renderer how to indent,
     *                                  only change this if the page source doesn't
     *                                  look right. (default 1)
     *
     *  @param  int     $output_debug   This option turns the debugging output on
     *                                  or off. (default 0)
     *
     *  @return string                  The generated HTML output for the table
     */
    function render($indent_level = 1, $output_debug=0)
    {
        $menu = html_div("widget");

        if($this->title) {
            $title = html_h3($this->title);
            $title->set_class('widget-title');
            $menu->add($title);
        }
            
        foreach($this->_content as $cont) {
            $div = html_div('widget-item');
            $div->add($cont);
            $menu->add($div);
        }

        return $menu->render($indent_level, $output_debug);
    }

//}}}

}

/**
  * The Header block for the default maintain skin
  *
  * @package    skins
  * @subpackage maintain
  */
class HeaderForm extends FormContent
{

//{{{   form_init_elements()

    /**
     *  This method is what creates the form object. 
     */
    function form_init_elements() 
    {
        global $user;
        
        $zoneids = $user->zones();
        $zones = array();
        foreach($zoneids as $zoneid) {
            $zone = new Zone($zoneid);
            $zones[$zone->name] = $zoneid;
        }
        $zone_list = new FEListBox("zoneselect", false, "200px");
        $zone_list->set_list_data($zones); //$user->zones()
        $this->add_element($zone_list);
        $this->set_action($_SERVER['SCRIPT_NAME']);
    }

//}}}
//{{{   form_init_data()

    /**
     *  This method is what sets the initial value for them form.
     */
    function form_init_data()
    {
        $this->set_element_value("zoneselect",$_SESSION['zone']);
    }

//}}}
//{{{   form()

    /**
     *  This is the method that lays out the form for rendering.
     */
    function form()
    {
        return container($this->element_form("zoneselect"),$this->add_action("Go!"));
    }

//}}}
//{{{   form_action()

    /**
     *  This is the method that runs when the form submission passes validation.
     *  In this instance it puts them to index where their new zone will take 
     *  effect.
     */
    function form_action()
    {
        global $conf;
        header("Location: ".web_root()."index.php");
    }

//}}}
//{{{ form_backend_validation()

    /**
     *  There really isn't much validation to run at the moment.  This may change.
     *
     *  @return boolean validation success status
     */
    function form_backend_validation()
    {
        return true;
    }

//}}}

}

/**
 *  The login layout for the login page
 *
 *  @package    skins
 *  @subpackage maintain
 */
class LoginLayout extends Layout
{

//{{{   LoginLayout()

    /**
     *  Constructor
     *
     *  @param  String  $title  The page title
     *  @param  Const   $render The render method (Default HTML)
     */
    function LoginLayout($title, $render = HTML)
    {
        $this->Layout($title, $render);
        $this->add_css_link(web_root()."skins/maintain/style.css");
    }

//}}}
//{{{   body_content()

    /**
     *  This method is what actually puts the layout together
     */
    function body_content()
    {
        $this->add($this->login_body());
    }
   
//}}}
//{{{   login_body()

    /**
     *  This helper function builds the login div.
     *
     *  @return $div    The div that contains the form
     */
    function login_body()
    {
        $div = html_div();

        $div->add(html_br(2));

        $login = html_div("loginform");
        $login->add($this->content);
        $div->add($login);

        
        return $div;
    }

//}}}

}

/**
 * The install layout for the installation page
 *
 * @package     skins
 * @subpackage  maintain
 */
class InstallLayout extends LoginLayout
{

// {{{ body_content()

    function body_content()
    {
        $this->add($this->content);
    }

// }}}

}

/**
 *  The footer widget for the main layout
 *  
 *  @package    skins
 *  @subpackage maintain
 */
class FooterWidget extends BaseWidget
{

//{{{ vars

    var $class;

//}}}
//{{{   constructor()
    
    /**
     *  Constructor
     *
     *  @param String   $class  The name of the div class
     */
    function FooterWidget($class = 'footer')
    {
        $this->class = $class;
    }

//}}}
//{{{   render()

    /**
     *  This method renders the object into HTML code
     *
     *  @param  int $indent_level   The level of indentation, makes pretty code.
     *  @param  int $debug          Set to 1 to enable debugging.
     *
     *  @return String              The object in HTML representation.
     */
    function render($indent_level = 1, $debug = 0)
    {
        global $user;
        
        $div = html_div($this->class);
        
        // get module additions to navlinks
        $div = Module::apply_filter('navlinks', $div);
        
        return $div->render($indent_level, $debug);
    }

//}}}

}

?>
